echo on
% Programa Ejemplo_04_fsolve que utiliza el comado fsolve
% Solution to the problem posed in Example_04. It calculates the
% equilibrium concentration of the components of a system of two
% reversible chemical reactions using the optimization command fsolve.
% Input data
% c0	= Initial concentration
% K1       	= Equilibrium constant of the first Equation
% K2       	= Equilibrium constant of the second Equation
% rho      	= Relaxation factor
% x0       	= Vector of initial guesses
% tol	= Tolerance
echo off
clear
clc
c0 = input(' Vector of initial concentration of A, B, C, and D = ');
K1 = input(' 2A + B = C K1 = ');
K2 = input(' A + D = C K2 = ');
disp('Graph of the function')
x1=(0.1:0.01:0.5);
x2=(0.1:0.01:0.5);
[X1,X2]=meshgrid(x1,x2);
ca = c0(1) - 2*X1*c0(2) - X2*c0(4);
cb = (1 - X1)*c0(2);
cc = c0(3) + X1*c0(2) + X2*c0(4);
cd = (1 - X2)*c0(4);
Z1 = cc./(ca.^2)./cb - K1;
Z2 = cc./ca./cd - K2;
mesh(X1,X2,Z1);
hold on
mesh(X1,X2,Z2);
pause
disp('Zero Level Curves')
figure(2)
contour(X1,X2,Z1,[0 0]);
hold on
contour(X1,X2,Z2,[0 0]);
grid on
hold off
pause
disp('Root estimation')
disp('Click on the intersection of the curves')
[x1e,x2e] = ginput(1)
pause
repeat = 1;
while repeat
    x0 = input('\n\n Vector column of initial guesses = ');
    rho = input(' Relaxation factor = ');
    tol = input (' Tolerance = ');
    % Solution of the set of equations
    options=optimset('Display', 'iter'); % Option to display output
    [x,fval,exitflag,output,jacobian] = fsolve(@(x) Ex_04_func(x,c0,K1,K2),x0,options); % Call optimizer 
    % Display the results
    fprintf('\n Results :\n x1 = %6.4f , x2 = %6.4f\n',x)
%
    disp('Exitflag value'), exitflag
echo on
    % exitflag: Integer identifying the reason the algorithm terminated. 
    % The following lists the values of exitflag and the corresponding 
    % reasons the algorithm terminated.
    %   1  FSOLVE converged to a solution X.
    %   2  Change in X smaller than the specified tolerance.
    %   3  Change in the residual smaller than the specified tolerance.
    %   4  Magnitude of search direction smaller than the specified tolerance.
    %   0  Maximum number of function evaluations or iterations reached.
    %  -1  Algorithm terminated by the output function.
    %  -2  Algorithm seems to be converging to a point that is not a root.
    %  -3  Trust region radius became too small.
    %  -4  Line search cannot sufficiently decrease the residual along the
    %  current search direction.
echo off
%
    disp('Output values = '), output
echo on
    % output: Structure containing information about the optimization. 
    % The fields of the structure are
    %   iterations: Number of iterations taken
    %   funcCount: Number of function evaluations
    %   algorithm: Algorithm used
    %   cgiterations: Number of PCG iterations (large-scale algorithm only)
    %   stepsize: Final step size taken (medium-scale algorithm only)
    %   firstorderopt: Measure of first-order optimality (large-scale
    %   algorithm only).
    %   For large-scale problems, the first-order optimality is the infinity norm of 
    %   the gradient g = JTF (see Nonlinear Least-Squares).
echo off
    disp ('jacobian at the solution x = '), jacobian
    repeat = input(' Repeat the calculations (0 / 1) ? ');
end